/*
 * Copyright (c) 2012 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin
*/

var GLUM = {};
var GLUM_MODELVIEW;
var GLUM_PROJECTION;

function glumResize()
{
    if (GLUM.canvas.fullwindow) {
        GLUM.canvas.width = 0.95*document.body.clientWidth;
        GLUM.canvas.height = 0.95*document.body.clientHeight;
    }
    GLUM.gl.viewport(0, 0, GLUM.canvas.width, GLUM.canvas.height);
}

function glumReshapeFunc(reshapeFunc) {
    document.body.onresize=reshapeFunc;
}

function glumInitDisplayMode(cvname, displayMode)
{
    try {
        GLUM.canvas = document.getElementById(cvname, displayMode);
    } catch (error) {
        alert("glumInitDisplayMode: failed."); 
        return;
    }
    return(GLUM.canvas);
}

function glumInitCanvasSize(w, h)
{
    try {
        GLUM.gl = GLUM.canvas.getContext("experimental-webgl");
    } catch (error) {
        alert("glumInitCanvasSize: getContext failed."); 
        return;
    }
    GLUM.canvas.fullwindow = (w == 0 || w/document.body.clientWidth >= 0.99);
    glumResize();

    return(GLUM.gl);
}

function glumInitShaders(vsname, fsname)
{
    var vsource = document.getElementById(vsname).text;
    var vs = GLUM.gl.createShader(GLUM.gl.VERTEX_SHADER);
    GLUM.gl.shaderSource(vs, vsource);
    GLUM.gl.compileShader(vs);
    if (!GLUM.gl.getShaderParameter(vs, GLUM.gl.COMPILE_STATUS)) {
        alert(GLUM.gl.getShaderInfoLog(vs));
        GLUM.gl.deleteShader(vs);
        return;
    }
    
    var fsource = document.getElementById(fsname).text;
    var fs = GLUM.gl.createShader(GLUM.gl.FRAGMENT_SHADER);
    GLUM.gl.shaderSource(fs, fsource);
    GLUM.gl.compileShader(fs);
    if (!GLUM.gl.getShaderParameter(fs, GLUM.gl.COMPILE_STATUS)) {
        alert(GLUM.gl.getShaderInfoLog(fs));
        GLUM.gl.deleteShader(vs);
        GLUM.gl.deleteShader(fs);
        return;
    }

    GLUM.spd = GLUM.gl.createProgram();
    GLUM.gl.attachShader(GLUM.spd, vs);
    GLUM.gl.attachShader(GLUM.spd, fs);
    GLUM.gl.linkProgram(GLUM.spd);
    if (!GLUM.gl.getProgramParameter(GLUM.spd, GLUM.gl.LINK_STATUS)) {
        alert("glumInitShaders: linkProgram failed.");
        GLUM.gl.detachShader(GLUM.spd, vs);
        GLUM.gl.detachShader(GLUM.spd, fs);
        GLUM.gl.deleteShader(vs);
        GLUM.gl.deleteShader(fs);
        GLUM.gl.deleteProgram(GLUM.spd);
        return;
    }

    GLUM.gl.useProgram(GLUM.spd);
    return(GLUM.spd);
}

function glumMatrixMode(mat) {
    GLUM.CTM = mat;
}

function glumLoadIdentity()
{
  GLUM.CTM[0] = 1.0;
  GLUM.CTM[1] = 0.0;
  GLUM.CTM[2] = 0.0;
  GLUM.CTM[3] = 0.0;

  GLUM.CTM[4] = 0.0;
  GLUM.CTM[5] = 1.0;
  GLUM.CTM[6] = 0.0;
  GLUM.CTM[7] = 0.0;

  GLUM.CTM[8] = 0.0;
  GLUM.CTM[9] = 0.0;
  GLUM.CTM[10] = 1.0;
  GLUM.CTM[11] = 0.0;

  GLUM.CTM[12] = 0.0;
  GLUM.CTM[13] = 0.0;
  GLUM.CTM[14] = 0.0;
  GLUM.CTM[15] = 1.0;
}

function glumPushMatrix()
{
    if (GLUM.CTM == GLUM_MODELVIEW) {
        GLUM.ModelViewStack.push(new Float32Array(GLUM_MODELVIEW));
    } else {
        GLUM.ProjectionStack.push(new Float32Array(GLUM_PROJECTION));
    }
}

function glumPopMatrix()
{
    if (GLUM.CTM == GLUM_MODELVIEW) {
        GLUM.CTM = GLUM_MODELVIEW = GLUM.ModelViewStack.pop();
    } else {
        GLUM.CTM = GLUM_PROJECTION = GLUM.ProjectionStack.pop();
    }
}

function glumShaderMatrix() {
    if (GLUM.CTM == GLUM_MODELVIEW) {
        GLUM.gl.uniformMatrix4fv(GLUM.mvMatloc, false, GLUM_MODELVIEW);
    } else {
        GLUM.gl.uniformMatrix4fv(GLUM.pMatloc, false, GLUM_PROJECTION);
    }
}

function glumInit()
{
  if (GLUM.canvas == undefined || GLUM.gl == undefined || GLUM.spd == undefined) {
      alert("glumInit(): glumInitDisplayMode() or glumInitCanvasSize() or glumInitShaders() not called prior to calling glumInit()");
      return;
  }
    
  // as per GLSL, matrices are column major
  GLUM_MODELVIEW = new Float32Array(16);
  GLUM_PROJECTION = new Float32Array(16);

  GLUM.mvMatloc = GLUM.gl.getUniformLocation(GLUM.spd, "ModelViewMatrix");
  GLUM.pMatloc = GLUM.gl.getUniformLocation(GLUM.spd, "ProjectionMatrix");
  GLUM.CTM;
  GLUM.ModelViewStack = [];
  GLUM.ProjectionStack = [];

  glumMatrixMode(GLUM_MODELVIEW);
  glumLoadIdentity();
  glumShaderMatrix();

  // add aliases for OES_vertex_array_object extensions
  try {
      GLUM.gl.ext = GLUM.gl.getExtension("OES_vertex_array_object");
      GLUM.gl.createVertexArray = function() { return(GLUM.gl.ext.createVertexArrayOES()); };
      GLUM.gl.deleteVertexArray = function(arrayObject) { return(GLUM.gl.ext.deleteVertexArrayOES(arrayObject)); };
      GLUM.gl.isVertexArray = function(arrayObject) { return(GLUM.gl.ext.isVertexArrayOES(arrayObject)); };
      GLUM.gl.bindVertexArray = function(arrayObject) { return(GLUM.gl.ext.bindVertexArrayOES(arrayObject)); };
  } catch(error){}
    
  return;
}

function glumLookAt(eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ)
{
  // first compute Camera Coordinate Frame by performing Gram-Schmidt
  // Orthonormalization
  wX = eyeX - centerX;
  wY = eyeY - centerY;
  wZ = eyeZ - centerZ;
  if (!(wX | wY | wZ)) {
    glumLoadIdentity(GLUM_MODELVIEW);
    return;
  }
  wNorm = 1.0/Math.sqrt(wX*wX+wY*wY*wZ*wZ);
  wX *= wNorm;
  wY *= wNorm;
  wZ *= wNorm;

  uX = upY*wZ - upZ*wY;
  uY = upZ*wX - upX*wZ;
  uZ = upX*wY - upY*wX;
  uNorm = 1.0/Math.sqrt(uX*uX+uY*uY+uZ*uZ);
  uX *= uNorm;
  uY *= uNorm;
  uZ *= uNorm;
        
  vX = wY*uZ - wZ*uY;
  vY = wZ*uX - wX*uZ;
  vZ = wX*uY - wY*uX;
  // already normalized

  // 1st col
  GLUM.CTM[0] = uX;
  GLUM.CTM[1] = vX;
  GLUM.CTM[2] = wX;
  GLUM.CTM[3] = 0.0;
        
  // 2nd col
  GLUM.CTM[4] = uY;
  GLUM.CTM[5] = vY;
  GLUM.CTM[6] = wY;
  GLUM.CTM[7] = 0.0;
        
  // 3rd col
  GLUM.CTM[8] = uZ;
  GLUM.CTM[9] = vZ;
  GLUM.CTM[10] = wZ;
  GLUM.CTM[11] = 0.0;
        
  // 4th col
  GLUM.CTM[12] = -(uX*eyeX+uY*eyeY+uZ*eyeZ);
  GLUM.CTM[13] = -(vX*eyeX+vY*eyeY+vZ*eyeZ);
  GLUM.CTM[14] = -(wX*eyeX+wY*eyeY+wZ*eyeZ);
  GLUM.CTM[15] = 1.0;

  // don't want to copy over to shader automatically as
  // we will likely make more changes to the matrix
}

function glumTranslatef(x, y, z)
{
    GLUM.CTM[12] += GLUM.CTM[0]*x + GLUM.CTM[4]*y + GLUM.CTM[8]*z;
    GLUM.CTM[13] += GLUM.CTM[1]*x + GLUM.CTM[5]*y + GLUM.CTM[9]*z;
    GLUM.CTM[14] += GLUM.CTM[2]*x + GLUM.CTM[6]*y + GLUM.CTM[10]*z;
}

function glumScalef(x, y, z)
{
    GLUM.CTM[0] *= x;
    GLUM.CTM[1] *= x;
    GLUM.CTM[2] *= x;

    GLUM.CTM[4] *= y;
    GLUM.CTM[5] *= y;
    GLUM.CTM[6] *= y;

    GLUM.CTM[7] *= z;
    GLUM.CTM[8] *= z;
    GLUM.CTM[9] *= z;
}

function glumRotatef(angle, x, y, z)
{
    // Rotation by Rodrigues Formula in matrix form
    var c = Math.cos(angle*Math.PI/180.0);
    var s = Math.sin(angle*Math.PI/180.0);
    var oc = 1.0 - c;

    aNorm = 1.0/Math.sqrt(x*x+y*y+z*z);
    x *= aNorm;
    y *= aNorm;
    z *= aNorm;

    var r00 = oc*x*x+c;
    var r01 = oc*x*y+s*z;
    var r02 = oc*x*z-s*y;

    var r10 = oc*x*y-s*z;
    var r11 = oc*y*y+c;
    var r12 = oc*y*z+s*x;

    var r20 = oc*x*z+s*y;
    var r21 = oc*y*z-s*x;
    var r22 = oc*z*z+c;

    var m00 = GLUM.CTM[0];
    var m01 = GLUM.CTM[4];
    var m02 = GLUM.CTM[8];
    
    var m10 = GLUM.CTM[1];
    var m11 = GLUM.CTM[5];
    var m12 = GLUM.CTM[9];
    
    var m20 = GLUM.CTM[2];
    var m21 = GLUM.CTM[6];
    var m22 = GLUM.CTM[10];
    
    GLUM.CTM[0] = m00*r00 + m01*r10 + m02*r20;
    GLUM.CTM[4] = m00*r01 + m01*r11 + m02*r21;
    GLUM.CTM[8] = m00*r02 + m01*r12 + m02*r22;

    GLUM.CTM[1] = m10*r00 + m11*r10 + m12*r20;
    GLUM.CTM[5] = m10*r01 + m11*r11 + m12*r21;
    GLUM.CTM[9] = m10*r02 + m11*r12 + m12*r22;

    GLUM.CTM[2] = m20*r00 + m21*r10 + m22*r20;
    GLUM.CTM[6] = m20*r01 + m21*r11 + m22*r21;
    GLUM.CTM[10] = m20*r02 + m21*r12 + m22*r22;
};

function glumFrustum(left, right, bottom, top, nearVal, farVal)
{
  var r_l = right - left;
  var t_b = top - bottom;
  var n_f = nearVal - farVal; // near and far assumed positive distances
        
  // 1st col
  GLUM.CTM[0] = 2.0*nearVal/r_l;
  GLUM.CTM[1] = 0.0;
  GLUM.CTM[2] = 0.0;
  GLUM.CTM[3] = 0.0;

  // 2nd col
  GLUM.CTM[4] = 0.0;
  GLUM.CTM[5] = 2.0*nearVal/t_b;
  GLUM.CTM[6] = 0.0;
  GLUM.CTM[7] = 0.0;

  // 3rd col
  GLUM.CTM[8] = (right+left)/r_l;
  GLUM.CTM[9] = (top+bottom)/t_b;
  GLUM.CTM[10] = (nearVal+farVal)/n_f;
  GLUM.CTM[11] = -1.0;

  // 4th col
  GLUM.CTM[12] = 0.0;
  GLUM.CTM[13] = 0.0;
  GLUM.CTM[14] = 2.0*farVal*nearVal/n_f;
  GLUM.CTM[15] = 0.0;
}

function glumPerspective(fovy, aspect, zNear, zFar)
{
  /* tan(fovy/2) = top/near => near = top/tan(fovy/2)
     right/top = aspect     => right = top * aspect
     plugging these into the frustum matrix gives us the
     following matrix
  */

  var n_f = zNear - zFar; // near and far assumed positive distances
  var cotan_fovy_2_rad = 1.0/Math.tan(fovy*Math.PI/360.0);
        
  // 1st col
  GLUM.CTM[0] = (1.0/aspect)*cotan_fovy_2_rad;
  GLUM.CTM[1] = 0.0;
  GLUM.CTM[2] = 0.0;
  GLUM.CTM[3] = 0.0;

  // 2nd col
  GLUM.CTM[4] = 0.0;
  GLUM.CTM[5] = cotan_fovy_2_rad;
  GLUM.CTM[6] = 0.0;
  GLUM.CTM[7] = 0.0;

  // 3rd col
  GLUM.CTM[8] = 0.0;
  GLUM.CTM[9] = 0.0;
  GLUM.CTM[10] = (zNear+zFar)/n_f;
  GLUM.CTM[11] = -1.0;

  // 4th col
  GLUM.CTM[12] = 0.0;
  GLUM.CTM[13] = 0.0;
  GLUM.CTM[14] = 2.0*zFar*zNear/n_f;
  GLUM.CTM[15] = 0.0;
}

function glumOrtho(left, right, bottom, top, nearVal, farVal)
{
  const EPSILON=0.1;
  var r_l = right - left;
  var t_b = top - bottom;
  var n_f = nearVal - farVal; // assume f > n, so instead of 1/(n-f),
                              // we use 1/(f-n) in constructing the
                              // projection matrix

  // 1st col
  GLUM.CTM[0] = 2.0/r_l;
  GLUM.CTM[1] = 0.0;
  GLUM.CTM[2] = 0.0;
  GLUM.CTM[3] = 0.0;

  // 2nd col
  GLUM.CTM[4] = 0.0;
  GLUM.CTM[5] = 2.0/t_b;
  GLUM.CTM[6] = 0.0;
  GLUM.CTM[7] = 0.0;

  // 3rd col
  GLUM.CTM[8] = 0.0;
  GLUM.CTM[9] = 0.0;
  GLUM.CTM[10] = 2.0/n_f;
  GLUM.CTM[11] = 0.0;

  // 4th col
  GLUM.CTM[12] = -(right+left)/r_l;
  GLUM.CTM[13] = -(top+bottom)/t_b;
  GLUM.CTM[14] = (farVal+nearVal)/n_f;
  GLUM.CTM[15] = 1.0;
}

function glumOrtho2D(left, right, bottom, top)
{
  glumOrtho(left, right, bottom, top, -1.0, 1.0);
}
