/*
 * Copyright (c) 2012 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Authors: Sugih Jamin
*/

var sample_spd;
var sample_canvas;
var sample_gl;

function sample_initWorld() {
    var vertices = new Float32Array([
       0.25*sample_canvas.width,  0.75*sample_canvas.height,
       0.0, 0.0, 0.0,
       0.75*sample_canvas.width,  0.25*sample_canvas.height,
       0.0, 0.0, 0.0
    ]);

    var vbod = sample_gl.createBuffer();
    sample_gl.bindBuffer(sample_gl.ARRAY_BUFFER, vbod);
    sample_gl.bufferData(sample_gl.ARRAY_BUFFER, vertices, sample_gl.DYNAMIC_DRAW);

    var vPos = sample_gl.getAttribLocation(sample_spd, "vPos");
    sample_gl.enableVertexAttribArray(vPos);
    sample_gl.vertexAttribPointer(vPos, 2, sample_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 0);
    var vCol = sample_gl.getAttribLocation(sample_spd, "vCol");
    sample_gl.enableVertexAttribArray(vCol);
    sample_gl.vertexAttribPointer(vCol, 3, sample_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 2*Float32Array.BYTES_PER_ELEMENT);
}

function sample_drawWorld()
{
    sample_gl.clearColor(1.0, 1.0, 1.0, 0.0);
    sample_gl.clear(sample_gl.COLOR_BUFFER_BIT | sample_gl.DEPTH_BUFFER_BIT);
    sample_gl.enable(sample_gl.DEPTH_TEST);

    glumMatrixMode(GLUM_PROJECTION);
    glumLoadIdentity();
    glumOrtho2D(0.0, sample_canvas.width, 0.0, sample_canvas.height);
    glumShaderMatrix();

    sample_gl.drawArrays(sample_gl.LINES, 0, 2);

    sample_gl.flush();
}

function sample_reshape()
{
    glumResize();

    var vertices = new Float32Array([
       0.25*sample_canvas.width,  0.75*sample_canvas.height,
       0.0, 0.0, 0.0,
       0.75*sample_canvas.width,  0.25*sample_canvas.height,
       0.0, 0.0, 0.0
    ]);
    sample_gl.bufferData(sample_gl.ARRAY_BUFFER, vertices, sample_gl.DYNAMIC_DRAW);

    sample_drawWorld();
}

function sample_main(cvname, vsname, fsname)
{
    sample_canvas = glumInitDisplayMode(cvname, { alpha: false, depth: false });
    //sample_canvas = glumInitDisplayMode("sample", { alpha: false, depth: false });
    //sample_gl = glumInitCanvasSize(sample_canvas.width, sample_canvas.height);
    sample_gl = glumInitCanvasSize(0, 0);
    sample_spd = glumInitShaders(vsname, fsname);
    //sample_spd = glumInitShaders("minvs", "minfs");
    glumInit();

    glumReshapeFunc(sample_reshape);

    sample_initWorld();
    sample_drawWorld();
}
